/*
Written by "Joseph Miller" <joseph@tidetamerboatlifts.com>
*/

#include <windows.h>
#include <stdarg.h>
#include <winsock2.h>
#include "../config.h"
#include "ttplayer.h"
#include "../bswap.h"
#include "../libvo/w32_common.h"
#include "../osdep/keycodes.h"
#include "../mplayer.h"
//#include </usr/include/w32api/commdlg.h>

#warning WINAPI - Need to #include <commdlg.h> and fix linking
#define WINELIB_NAME_AW(func) func##A
BOOL  WINAPI GetOpenFileNameA(LPOPENFILENAMEA ofn);
#define GetOpenFileName WINELIB_NAME_AW(GetOpenFileName)

#define IDC_OPEN 1001
#define IDC_PLAY 1002
#define IDC_STOP 1003
#define IDC_BEGIN 1004
#define IDC_BACK 1005
#define IDC_FORW 1006

int ttplayer_Play = 0;

HWND hwnd_Main = NULL;
HWND hwnd_Open = NULL;
HWND hwnd_Play = NULL;
HWND hwnd_Begin = NULL;
HWND hwnd_Back = NULL;
HWND hwnd_Forw = NULL;
HWND hwnd_Stop = NULL;

HBITMAP hbm_Open = NULL;
HBITMAP hbm_Play = NULL;
HBITMAP hbm_Begin = NULL;
HBITMAP hbm_Back = NULL;
HBITMAP hbm_Forw = NULL;
HBITMAP hbm_Stop = NULL;
HBITMAP hbm_Pause = NULL;

char tmpbuff[TT_TMPBUFF_SIZE];

char szClassName[ ] = "Player";
HINSTANCE g_hInst;
char ttplayer_filename[TTFILENAMESIZE] = "";
int instring(char * s, char c);
int rcpy(char * s, char * d, int len);
int winError(int err);
char * about_program_message();
int help();

/* Declare Windows procedure */
LRESULT CALLBACK WindowProcedure(HWND, UINT, WPARAM, LPARAM);


int ttplayer_init() {
	int err;
	HMENU menu;
	#ifdef TTPLAYERDEBUG
		printf("GUI::Initializing ttplayer\n");
	#endif

	WNDCLASSEX wincl;
	g_hInst = GetModuleHandle(NULL);

	//ttplayer_filename[0] = '\0';

	/* The Window structure */
    wincl.hInstance = g_hInst;
    wincl.lpszClassName = szClassName;
    wincl.lpfnWndProc = WindowProcedure;      /* This function is called by windows */
    wincl.style = CS_DBLCLKS;                 /* Catch double-clicks */
    wincl.cbSize = sizeof(WNDCLASSEX);

    /* Use default icon and mouse-pointer */
    wincl.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon(NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor(NULL, IDC_ARROW);
    wincl.lpszMenuName = NULL; /* No menu */
    wincl.cbClsExtra = 0;                      /* No extra bytes after the window class */
    wincl.cbWndExtra = 0;                      /* structure or the window instance */
    /* Use light-gray as the background of the window */
    //wincl.hbrBackground = (HBRUSH) GetStockObject(LTGRAY_BRUSH);
    wincl.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);

    /* Register the window class, if fail quit the program */
    if(!RegisterClassEx(&wincl)) return 0;

    /* The class is registered, let's create the program*/
    hwnd_Main = CreateWindowEx(
           WS_EX_CONTROLPARENT,                   /* Extended possibilites for variation */
           szClassName,         /* Classname */
           "Player - LAF",         /* Title Text */
           WS_OVERLAPPEDWINDOW, /* default window */
           CW_USEDEFAULT,       /* Windows decides the position */
           CW_USEDEFAULT,       /* where the window ends up on the screen */
           199,                 /* The programs width */
           90,                 /* and height in pixels */
           HWND_DESKTOP,        /* The window is a child-window to desktop */
           NULL,                /* No menu */
           g_hInst,       /* Program Instance handler */
           NULL                 /* No Window Creation data */
           );

    /* Make the window visible on the screen */
    ShowWindow(hwnd_Main, SW_SHOW);

    menu = LoadMenu(g_hInst, MAKEINTRESOURCE(MAINMENU));
    SetMenu(hwnd_Main, menu);

    /*MSG msg;
    while (PeekMessage(&msg, NULL, 0, 0,PM_REMOVE))
    {
		TranslateMessage(&msg);
        DispatchMessage(&msg);
    }*/
}

// destroy window
int ttplayer_destroy() {
	#ifdef TTPLAYERDEBUG
		printf("GUI::Destroying ttplayer\n");
	#endif

	MSG messages;

	PostMessage(hwnd_Main, WM_DESTROY, 0, 0);

	// clear out all messages
	while(GetMessage(&messages, NULL, 0, 0))
	{
		/* Translate virtual-key messages into character messages */
		TranslateMessage(&messages);
		/* Send message to WindowProcedure */
		DispatchMessage(&messages);
	}

	/* The program return-value is 0 - The value that PostQuitMessage( ) gave */
	return messages.wParam;
}

// set play indicator
int ttplayer_set_status(int playval) {
	switch (playval) {

		case TTPLAY_STOP:
			SendMessage(hwnd_Play, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Play);
			ttplayer_Play = playval;
		break;

		case TTPLAY_PAUSE:
			if ( ttplayer_Play != TTPLAY_STOP ) {
				SendMessage(hwnd_Play, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Play);
				ttplayer_Play = playval;
			}
		break;

		case TTPLAY_PLAY:
			SendMessage(hwnd_Play, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Pause);
			ttplayer_Play = playval;
		break;

		case TTPLAY_RESTART:
			if ( ttplayer_Play == TTPLAY_PLAY ) {
				ttplayer_Play = playval;
			}
		break;

		default:
			#ifdef TTPLAYERDEBUG
			printf("GUI::ttplayer_set_status(%d) = invalid status\n", playval);
			#endif
		break;
	}
}

// open a file
int ttplayer_fileopen() {
	OPENFILENAME ofn;

	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hwnd_Main;
	ofn.lpstrFilter = "All Files (*.*)\0*.*\0\0";
	ofn.lpstrDefExt = "*.*";
	ofn.lpstrFile = ttplayer_filename;
	ofn.nMaxFile = TTFILENAMESIZE;
	ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST;

	if ( GetOpenFileName(&ofn) ) {
		// successful

		//MessageBox(NULL, ttplayer_filename, "open()...", 0);

		//ttplayer_filename[0] = '\0';
		ttplayer_set_status(TTPLAY_PLAY);
	} else {
		// set status to stop
		ttplayer_set_status(TTPLAY_STOP);
	}



	#ifdef TTPLAYERDEBUG
		//strcpy(ttplayer_filename, "test.wmv");
	#endif
}

// copy a string, but reverse-style so that we don't clobber
int rcpy(char * s, char * d, int len) {
	int i;
	for(i=len;i>=0;i--) {
		d[i] = s[i];
	}

	return 0;
}

// searches a string for a character
int instring(char * s, char c) {
	int i = 0;
	while ( s[i] != '\0' ) {
		if ( s[i] == c )
			return 1;
		i++;
	}

	return 0;
}

/*// message box with formatting
int messageboxf(const char * title, const char * fmt, ...) {
}*/

int winError(int err) {

	LPVOID lpMsgBuf;

	if ( err == 0 ) {
		err = GetLastError();
	}


	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL,
	err,
	MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0, NULL);

	MessageBox(NULL, lpMsgBuf, "Error!", 0);

	#ifdef TTPLAYERDEBUG
	printf("Error: ");
	printf(lpMsgBuf);
	printf("\n");
	#endif

	LocalFree(lpMsgBuf);

	return err;
}

// display message about program
char * about_program_message() {
	// make sure we have null terminating character
	bzero(tmpbuff, TT_TMPBUFF_SIZE);
	// copy about string
	snprintf(tmpbuff, TT_TMPBUFF_SIZE-1, "MPlayer - The Movie Player");

	return tmpbuff;
}

// open the help documentation
int help() {

	ShellExecute(NULL, "open", "http://www.mplayerhq.hu/DOCS/HTML/en/index.html", NULL, NULL, SW_SHOWNORMAL);

	return 0;
}

/* This function is called by the Windows function DispatchMessage( ) */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)                  /* handle the messages */
    {
           case WM_CREATE:
              //MessageBox (NULL, "Creating Window" , "Message", 0);

	      //break;
              // ========== Open ============
              hwnd_Open = CreateWindowEx(
                WS_EX_STATICEDGE,
                "BUTTON",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP | WS_GROUP | BS_BITMAP,
                5,5,26,26,hwnd,(void *) IDC_OPEN,g_hInst,NULL
              );
              hbm_Open = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_OPEN));
              SendMessage(hwnd_Open, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Open);

              // ========== Begin ===========
              hwnd_Begin = CreateWindowEx(
                WS_EX_STATICEDGE,
                "BUTTON",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP | BS_BITMAP,
                5*2+26*1,5,26,26,hwnd,(void *) IDC_BEGIN,g_hInst,NULL
              );
              hbm_Begin = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_BEGIN));
              SendMessage(hwnd_Begin, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Begin);

              // ========== Back ===========
              hwnd_Back = CreateWindowEx(
                WS_EX_STATICEDGE,
                "BUTTON",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP | BS_BITMAP,
                5*3+26*2,5,26,26,hwnd,(void *) IDC_BACK,g_hInst,NULL
              );
              hbm_Back = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_BACK));
              SendMessage(hwnd_Back, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Back);

              // ========== Stop ===========
              hwnd_Stop = CreateWindowEx(
                WS_EX_STATICEDGE,
                "BUTTON",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP | BS_BITMAP,
                5*4+26*3,5,26,26,hwnd,(void *) IDC_STOP,g_hInst,NULL
              );
              hbm_Stop = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_STOP));
              SendMessage(hwnd_Stop, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Stop);

              // ========== Play ===========
              hwnd_Play = CreateWindowEx(
                WS_EX_STATICEDGE,
                "BUTTON",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP | BS_BITMAP,
                5*5+26*4,5,26,26,hwnd,(void *) IDC_PLAY,g_hInst,NULL
              );
              hbm_Play = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_PLAY));
              SendMessage(hwnd_Play, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Play);

              // ========== Forward ===========
              hwnd_Forw = CreateWindowEx(
                WS_EX_STATICEDGE,
                "BUTTON",
                NULL,
                WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP | BS_BITMAP,
                5*6+26*5,5,26,26,hwnd,(void *) IDC_FORW,g_hInst,NULL
              );
              hbm_Forw = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_FORW));
              SendMessage(hwnd_Forw, BM_SETIMAGE, IMAGE_BITMAP, (int)hbm_Forw);

	      // load other bitmaps
	      hbm_Pause = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_PAUSE));

           break;

           case WM_COMMAND:
	   	switch (wParam) {
			case CM_FILE_EXIT:
				PostMessage(hwnd_Main, WM_DESTROY, 0, 0);
			break;

			case CM_FILE_OPEN:
				ttplayer_fileopen();
			break;

			case CM_ABOUT:
				MessageBox(NULL, PROGRAM_ABOUT_MESSAGE, "About", 0);
			break;

			case CM_HELP_DOCUMENTATION:
				help();
			break;

			default:
			break;
		}
                switch (HIWORD(wParam)) {
                       // handle buttons
                       case BN_CLICKED:
                            if ( (HWND) lParam == hwnd_Open ) {
                                   // open button
                                   //MessageBox (NULL, "Open file here..." , "Open", 0);
				   ttplayer_fileopen();
                            } else if ( (HWND) lParam == hwnd_Begin ) {
                                   // go to beginning button
                                   //MessageBox (NULL, "Go to beginning..." , "Begin", 0);
				   if ( ttplayer_Play == TTPLAY_PLAY ) {
					//mplayer_put_key(KEY_HOME);
					ttplayer_set_status(TTPLAY_RESTART);
				   }
                            } else if ( (HWND) lParam == hwnd_Back ) {
                                   // go back a little bit
                                   //MessageBox (NULL, "Go back..." , "Back", 0);
				   if ( ttplayer_Play == TTPLAY_PLAY )
				   	mplayer_put_key(KEY_LEFT);
                            } else if ( (HWND) lParam == hwnd_Forw ) {
                                   // go forward a little bit
                                   //MessageBox (NULL, "Go forward..." , "Forward", 0);
				   if ( ttplayer_Play == TTPLAY_PLAY )
				   	mplayer_put_key(KEY_RIGHT);
                            } else if ( (HWND) lParam == hwnd_Play ) {
                                   // play
                                   //MessageBox (NULL, "Play file..." , "Play", 0);
				   if ( ttplayer_Play == TTPLAY_PLAY || ttplayer_Play == TTPLAY_PAUSE )
				   	mplayer_put_key((int)' ');
				   else
				   	ttplayer_set_status(TTPLAY_PLAY);
                            } else if ( (HWND) lParam == hwnd_Stop ) {
                                   // stop
                                   //MessageBox (NULL, "Stop playing..." , "Stop", 0);
				   ttplayer_set_status(TTPLAY_STOP);
                            }

                       break;

                       default:
                       break;
                }
           break;

           case WM_DESTROY:
		PostQuitMessage(0);        /* send a WM_QUIT to the message queue */

		if ( ttplayer_Play == TTPLAY_STOP ) {
			ttplayer_Play = TTPLAY_EXIT;
			exit_player( "Exit" );
		} else {
	   		mplayer_put_key(KEY_ESC);
		}

           break;

           default:                   /* for messages that we don't deal with */
           return DefWindowProc(hwnd, message, wParam, lParam);
    }
    return 0;
}
